@@ -18,9 +18,7 @@
 
 #include "config.h"
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE /* activate extra prototypes for glibc */
-#endif
+#include <openssl/opensslv.h>
 
 #include <sys/types.h>
 #include <sys/socket.h> /* For CMSG_* */
@@ -32,15 +30,35 @@
 # include <bstring.h>
 #endif
 
-#ifdef HAVE_ENDIAN_H
+#if defined(HAVE_ENDIAN_H)
 # include <endian.h>
+#elif defined(HAVE_SYS_ENDIAN_H)
+# include <sys/endian.h>
+#elif defined(HAVE_LIBKERN_OSBYTEORDER_H)
+# include <machine/endian.h>
+# include <libkern/OSByteOrder.h>
+
+# define htobe16(x) OSSwapHostToBigInt16(x)
+# define htole16(x) OSSwapHostToLittleInt16(x)
+# define be16toh(x) OSSwapBigToHostInt16(x)
+# define le16toh(x) OSSwapLittleToHostInt16(x)
+
+# define htobe32(x) OSSwapHostToBigInt32(x)
+# define htole32(x) OSSwapHostToLittleInt32(x)
+# define be32toh(x) OSSwapBigToHostInt32(x)
+# define le32toh(x) OSSwapLittleToHostInt32(x)
+
+# define htobe64(x) OSSwapHostToBigInt64(x)
+# define htole64(x) OSSwapHostToLittleInt64(x)
+# define be64toh(x) OSSwapBigToHostInt64(x)
+# define le64toh(x) OSSwapLittleToHostInt64(x)
+#else
+# error no compatible endian.h header found
 #endif
+
 #ifdef HAVE_MAILLOCK_H
 # include <maillock.h> /* For _PATH_MAILDIR */
 #endif
-#ifdef HAVE_PATHS_H
-# include <paths.h>
-#endif
 
 #ifdef HAVE_RPC_TYPES_H
 # include <rpc/types.h> /* For INADDR_LOOPBACK */
@@ -52,6 +70,7 @@
 # include <pam/pam_appl.h>
 #endif
 #endif
+
 #include <errno.h>
 
 /* chl */
@@ -64,8 +83,6 @@
 # define NETDB_INTERNAL (-1)
 #endif
 
-#include <openssl/opensslv.h> /* For OPENSSL_VERSION_NUMBER */
-
 #include "defines.h"
 
 #include "openbsd-compat.h"
