@@ -1,4 +1,4 @@
-/*	$OpenBSD: util.c,v 1.154 2021/06/14 17:58:16 eric Exp $	*/
+/*	$OpenBSD: util.c,v 1.159 2024/06/02 23:26:39 jsg Exp $	*/
 
 /*
  * Copyright (c) 2000,2001 Markus Friedl.  All rights reserved.
@@ -44,8 +44,6 @@
 #include "smtpd.h"
 #include "log.h"
 
-const char *log_in6addr(const struct in6_addr *);
-const char *log_sockaddr(struct sockaddr *);
 static int  parse_mailname_file(char *, size_t);
 
 int	tracing = 0;
@@ -750,22 +748,24 @@ parse_mailname_file(char *hostname, size_t len)
 	if ((fp = fopen(MAILNAME_FILE, "r")) == NULL)
 		return 1;
 
-	if ((buflen = getline(&buf, &bufsz, fp)) == -1)
-		goto error;
+	buflen = getline(&buf, &bufsz, fp);
+	fclose(fp);
+	if (buflen == -1) {
+		free(buf);
+		return 1;
+	}
 
 	if (buf[buflen - 1] == '\n')
 		buf[buflen - 1] = '\0';
 
-	if (strlcpy(hostname, buf, len) >= len) {
+	bufsz = strlcpy(hostname, buf, len);
+	free(buf);
+	if (bufsz >= len) {
 		fprintf(stderr, MAILNAME_FILE " entry too long");
-		goto error;
+		return 1;
 	}
 
 	return 0;
-error:
-	fclose(fp);
-	free(buf);
-	return 1;
 }
 
 int
@@ -861,7 +861,7 @@ int
 parse_table_line(FILE *fp, char **line, size_t *linesize,
     int *type, char **key, char **val, int *malformed)
 {
-	char	*keyp, *valp, *p;
+	char	*keyp, *valp;
 	ssize_t	 linelen;
 
 	*key = NULL;
@@ -891,16 +891,17 @@ parse_table_line(FILE *fp, char **line, size_t *linesize,
 		return 0;
 	}
 
-	if (*type == T_NONE) {
-		for (p = keyp; *p; p++) {
-			if (*p == ' ' || *p == '\t' || *p == ':') {
-				*type = T_HASH;
-				break;
-			}
+	if (*keyp == '[') {
+		if ((valp = strchr(keyp, ']')) == NULL) {
+			*malformed = 1;
+			return (0);
 		}
-		if (*type == T_NONE)
-			*type = T_LIST;
-	}
+		valp++;
+	} else
+		valp = keyp + strcspn(keyp, " \t:");
+
+	if (*type == T_NONE)
+		*type = (*valp == '\0') ? T_LIST : T_HASH;
 
 	if (*type == T_LIST) {
 		*key = keyp;
@@ -908,20 +909,11 @@ parse_table_line(FILE *fp, char **line, size_t *linesize,
 	}
 
 	/* T_HASH */
-	valp = keyp;
-	strsep(&valp, " \t:");
-	if (valp) {
-		while (*valp) {
-			if (!isspace((unsigned char)*valp) &&
-			    !(*valp == ':' &&
-			    isspace((unsigned char)*(valp + 1))))
-				break;
-			++valp;
-		}
-		if (*valp == '\0')
-			valp = NULL;
+	if (*valp != '\0') {
+		*valp++ = '\0';
+		valp += strspn(valp, " \t");
 	}
-	if (valp == NULL)
+	if (*valp == '\0')
 		*malformed = 1;
 
 	*key = keyp;
